package sun.print;

import javax.print.ServiceUIFactory;
import javax.print.attribute.HashAttributeSet;
import javax.print.attribute.standard.CopiesSupported;
import javax.print.attribute.standard.RequestingUserName;
import javax.print.attribute.standard.JobName;
import javax.print.attribute.AttributeSet;
import javax.print.attribute.standard.MediaSize;
import java.util.Locale;
import javax.print.attribute.standard.Sides;
import javax.print.attribute.standard.SheetCollate;
import javax.print.attribute.standard.PageRanges;
import javax.print.attribute.standard.OrientationRequested;
import javax.print.attribute.standard.MediaPrintableArea;
import javax.print.attribute.standard.Media;
import javax.print.attribute.standard.Fidelity;
import javax.print.attribute.standard.Chromaticity;
import javax.print.attribute.standard.Copies;
import javax.print.attribute.Attribute;
import javax.print.attribute.AttributeSetUtilities;
import javax.print.attribute.HashPrintServiceAttributeSet;
import javax.print.attribute.PrintServiceAttributeSet;
import javax.print.attribute.standard.ColorSupported;
import javax.print.attribute.PrintServiceAttribute;
import javax.print.event.PrintServiceAttributeListener;
import javax.print.DocPrintJob;
import javax.print.DocFlavor;
import java.io.OutputStream;
import javax.print.attribute.standard.MediaSizeName;
import javax.print.StreamPrintService;

public class PSStreamPrintService extends StreamPrintService implements SunPrinterJobService
{
    private static final Class[] suppAttrCats;
    private static int MAXCOPIES;
    private static final MediaSizeName[] mediaSizes;
    
    public PSStreamPrintService(OutputStream out) {
        super(out);
    }
    
    @Override
    public String getOutputFormat() {
        return "application/postscript";
    }
    
    @Override
    public DocFlavor[] getSupportedDocFlavors() {
        return PSStreamPrinterFactory.getFlavors();
    }
    
    @Override
    public DocPrintJob createPrintJob() {
        return new PSStreamPrintJob(this);
    }
    
    @Override
    public boolean usesClass(Class clazz) {
        return clazz == PSPrinterJob.class;
    }
    
    @Override
    public String getName() {
        return "Postscript output";
    }
    
    @Override
    public void addPrintServiceAttributeListener(PrintServiceAttributeListener printServiceAttributeListener) {
    }
    
    @Override
    public void removePrintServiceAttributeListener(PrintServiceAttributeListener printServiceAttributeListener) {
    }
    
    @Override
    public <T extends PrintServiceAttribute> T getAttribute(Class<T> clazz) {
        if (clazz == null) {
            throw new NullPointerException("category");
        }
        if (!PrintServiceAttribute.class.isAssignableFrom(clazz)) {
            throw new IllegalArgumentException("Not a PrintServiceAttribute");
        }
        if (clazz == ColorSupported.class) {
            return ColorSupported.SUPPORTED;
        }
        return null;
    }
    
    @Override
    public PrintServiceAttributeSet getAttributes() {
        HashPrintServiceAttributeSet attributeSet = new HashPrintServiceAttributeSet();
        attributeSet.add(ColorSupported.SUPPORTED);
        return AttributeSetUtilities.unmodifiableView(attributeSet);
    }
    
    @Override
    public boolean isDocFlavorSupported(DocFlavor docFlavor) {
        DocFlavor[] supportedDocFlavors = this.getSupportedDocFlavors();
        int i = 0;
        while (i < supportedDocFlavors.length) {
            if (docFlavor.equals(supportedDocFlavors[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    public Class<?>[] getSupportedAttributeCategories() {
        Class[] array = new Class[PSStreamPrintService.suppAttrCats.length];
        System.arraycopy(PSStreamPrintService.suppAttrCats, 0, array, 0, array.length);
        return array;
    }
    
    @Override
    public boolean isAttributeCategorySupported(Class<? extends Attribute> obj) {
        if (obj == null) {
            throw new NullPointerException("null category");
        }
        if (!Attribute.class.isAssignableFrom(obj)) {
            throw new IllegalArgumentException(obj + " is not an Attribute");
        }
        int i = 0;
        while (i < PSStreamPrintService.suppAttrCats.length) {
            if (obj == PSStreamPrintService.suppAttrCats[i]) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    public Object getDefaultAttributeValue(Class<? extends Attribute> obj) {
        if (obj == null) {
            throw new NullPointerException("null category");
        }
        if (!Attribute.class.isAssignableFrom(obj)) {
            throw new IllegalArgumentException(obj + " is not an Attribute");
        }
        if (!this.isAttributeCategorySupported(obj)) {
            return null;
        }
        if (obj == Copies.class) {
            return new Copies(1);
        }
        if (obj == Chromaticity.class) {
            return Chromaticity.COLOR;
        }
        if (obj == Fidelity.class) {
            return Fidelity.FIDELITY_FALSE;
        }
        if (obj == Media.class) {
            String country2 = Locale.getDefault().getCountry();
            if (country2 != null && (country2.equals("") || country2.equals(Locale.US.getCountry()) || country2.equals(Locale.CANADA.getCountry()))) {
                return MediaSizeName.NA_LETTER;
            }
            return MediaSizeName.ISO_A4;
        }
        else if (obj == MediaPrintableArea.class) {
            String country = Locale.getDefault().getCountry();
            float n = 0.5f;
            float w;
            float h;
            if (country != null && (country.equals("") || country.equals(Locale.US.getCountry()) || country.equals(Locale.CANADA.getCountry()))) {
                w = MediaSize.NA.LETTER.getX(25400) - 2.0f * n;
                h = MediaSize.NA.LETTER.getY(25400) - 2.0f * n;
            }
            else {
                w = MediaSize.ISO.A4.getX(25400) - 2.0f * n;
                h = MediaSize.ISO.A4.getY(25400) - 2.0f * n;
            }
            return new MediaPrintableArea(n, n, w, h, 25400);
        }
        else {
            if (obj == OrientationRequested.class) {
                return OrientationRequested.PORTRAIT;
            }
            if (obj == PageRanges.class) {
                return new PageRanges(1, Integer.MAX_VALUE);
            }
            if (obj == SheetCollate.class) {
                return SheetCollate.UNCOLLATED;
            }
            if (obj == Sides.class) {
                return Sides.ONE_SIDED;
            }
            return null;
        }
    }
    
    @Override
    public Object getSupportedAttributeValues(Class<? extends Attribute> obj, DocFlavor obj2, AttributeSet set) {
        if (obj == null) {
            throw new NullPointerException("null category");
        }
        if (!Attribute.class.isAssignableFrom(obj)) {
            throw new IllegalArgumentException(obj + " does not implement Attribute");
        }
        if (obj2 != null && !this.isDocFlavorSupported(obj2)) {
            throw new IllegalArgumentException(obj2 + " is an unsupported flavor");
        }
        if (!this.isAttributeCategorySupported(obj)) {
            return null;
        }
        if (obj == Chromaticity.class) {
            return new Chromaticity[] { Chromaticity.COLOR };
        }
        if (obj == JobName.class) {
            return new JobName("", null);
        }
        if (obj == RequestingUserName.class) {
            return new RequestingUserName("", null);
        }
        if (obj == OrientationRequested.class) {
            if (obj2 == null || obj2.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) || obj2.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                return new OrientationRequested[] { OrientationRequested.PORTRAIT, OrientationRequested.LANDSCAPE, OrientationRequested.REVERSE_LANDSCAPE };
            }
            return null;
        }
        else {
            if (obj == Copies.class || obj == CopiesSupported.class) {
                return new CopiesSupported(1, PSStreamPrintService.MAXCOPIES);
            }
            if (obj == Media.class) {
                Media[] array2 = new Media[PSStreamPrintService.mediaSizes.length];
                System.arraycopy(PSStreamPrintService.mediaSizes, 0, array2, 0, PSStreamPrintService.mediaSizes.length);
                return array2;
            }
            if (obj == Fidelity.class) {
                return new Fidelity[] { Fidelity.FIDELITY_FALSE, Fidelity.FIDELITY_TRUE };
            }
            if (obj == MediaPrintableArea.class) {
                if (set == null) {
                    return null;
                }
                MediaSize mediaSizeForName = (MediaSize)set.get(MediaSize.class);
                if (mediaSizeForName == null) {
                    Media media = (Media)set.get(Media.class);
                    if (media != null && media instanceof MediaSizeName) {
                        mediaSizeForName = MediaSize.getMediaSizeForName((MediaSizeName)media);
                    }
                }
                if (mediaSizeForName == null) {
                    return null;
                }
                MediaPrintableArea[] array = { null };
                float x = mediaSizeForName.getX(25400);
                float y = mediaSizeForName.getY(25400);
                float x2 = 0.5f;
                float y2 = 0.5f;
                if (x < 5.0f) {
                    x2 = x / 10.0f;
                }
                if (y < 5.0f) {
                    y2 = y / 10.0f;
                }
                array[0] = new MediaPrintableArea(x2, y2, x - 2.0f * x2, y - 2.0f * y2, 25400);
                return array;
            }
            else if (obj == PageRanges.class) {
                if (obj2 == null || obj2.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) || obj2.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                    return new PageRanges[] { new PageRanges(1, Integer.MAX_VALUE) };
                }
                return null;
            }
            else if (obj == SheetCollate.class) {
                if (obj2 == null || obj2.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) || obj2.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                    return new SheetCollate[] { SheetCollate.UNCOLLATED, SheetCollate.COLLATED };
                }
                return new SheetCollate[] { SheetCollate.UNCOLLATED };
            }
            else if (obj == Sides.class) {
                if (obj2 == null || obj2.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) || obj2.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                    return new Sides[] { Sides.ONE_SIDED, Sides.TWO_SIDED_LONG_EDGE, Sides.TWO_SIDED_SHORT_EDGE };
                }
                return null;
            }
            else {
                return null;
            }
        }
    }
    
    private boolean isSupportedCopies(Copies copies) {
        int value = copies.getValue();
        return value > 0 && value < PSStreamPrintService.MAXCOPIES;
    }
    
    private boolean isSupportedMedia(MediaSizeName mediaSizeName) {
        int i = 0;
        while (i < PSStreamPrintService.mediaSizes.length) {
            if (mediaSizeName.equals(PSStreamPrintService.mediaSizes[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    public boolean isAttributeValueSupported(Attribute attribute, DocFlavor obj, AttributeSet set) {
        if (attribute == null) {
            throw new NullPointerException("null attribute");
        }
        if (obj != null && !this.isDocFlavorSupported(obj)) {
            throw new IllegalArgumentException(obj + " is an unsupported flavor");
        }
        if (!this.isAttributeCategorySupported(attribute.getCategory())) {
            return false;
        }
        if (attribute.getCategory() == Chromaticity.class) {
            return attribute == Chromaticity.COLOR;
        }
        if (attribute.getCategory() == Copies.class) {
            return this.isSupportedCopies((Copies)attribute);
        }
        if (attribute.getCategory() == Media.class && attribute instanceof MediaSizeName) {
            return this.isSupportedMedia((MediaSizeName)attribute);
        }
        if (attribute.getCategory() == OrientationRequested.class) {
            if (attribute == OrientationRequested.REVERSE_PORTRAIT || (obj != null && !obj.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) && !obj.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE))) {
                return false;
            }
        }
        else if (attribute.getCategory() == PageRanges.class) {
            if (obj != null && !obj.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) && !obj.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                return false;
            }
        }
        else if (attribute.getCategory() == SheetCollate.class) {
            if (obj != null && !obj.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) && !obj.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                return false;
            }
        }
        else if (attribute.getCategory() == Sides.class && obj != null && !obj.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) && !obj.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
            return false;
        }
        return true;
    }
    
    @Override
    public AttributeSet getUnsupportedAttributes(DocFlavor obj, AttributeSet set) {
        if (obj != null && !this.isDocFlavorSupported(obj)) {
            throw new IllegalArgumentException("flavor " + obj + "is not supported");
        }
        if (set == null) {
            return null;
        }
        HashAttributeSet set2 = new HashAttributeSet();
        Attribute[] array = set.toArray();
        for (int i = 0; i < array.length; ++i) {
            try {
                Attribute attribute = array[i];
                if (!this.isAttributeCategorySupported(attribute.getCategory())) {
                    set2.add(attribute);
                }
                else if (!this.isAttributeValueSupported(attribute, obj, set)) {
                    set2.add(attribute);
                }
            }
            catch (ClassCastException ex) {}
        }
        if (set2.isEmpty()) {
            return null;
        }
        return set2;
    }
    
    @Override
    public ServiceUIFactory getServiceUIFactory() {
        return null;
    }
    
    @Override
    public String toString() {
        return "PSStreamPrintService: " + this.getName();
    }
    
    @Override
    public boolean equals(Object o) {
        return o == this || (o instanceof PSStreamPrintService && ((PSStreamPrintService)o).getName().equals(this.getName()));
    }
    
    @Override
    public int hashCode() {
        return this.getClass().hashCode() + this.getName().hashCode();
    }
    
    static {
        suppAttrCats = new Class[] { Chromaticity.class, Copies.class, Fidelity.class, JobName.class, Media.class, MediaPrintableArea.class, OrientationRequested.class, PageRanges.class, RequestingUserName.class, SheetCollate.class, Sides.class };
        PSStreamPrintService.MAXCOPIES = 1000;
        mediaSizes = new MediaSizeName[] { MediaSizeName.NA_LETTER, MediaSizeName.TABLOID, MediaSizeName.LEDGER, MediaSizeName.NA_LEGAL, MediaSizeName.EXECUTIVE, MediaSizeName.ISO_A3, MediaSizeName.ISO_A4, MediaSizeName.ISO_A5, MediaSizeName.ISO_B4, MediaSizeName.ISO_B5 };
    }
}
